package org.eco.als.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import org.eco.als.domain.bo.AircraftBo;
import org.eco.als.domain.vo.AircraftVo;
import org.eco.als.service.IAircraftService;
import org.eco.common.core.core.domain.CommonResult;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.log.annotation.Log;
import org.eco.common.log.enums.BusinessType;
import org.eco.common.security.utils.LoginHelper;
import org.eco.common.web.annotation.RepeatSubmit;
import org.eco.common.web.core.BaseController;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

/**
 * 机型机号Controller
 *
 * @author wgk
 * @date 2024-07-18
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/als/aircraft")
public class AircraftController extends BaseController {
    @Resource
    private IAircraftService aircraftService;

    /**
     * 查询机型机号列表
     */
    @SaCheckPermission("als:aircraft:list")
    @GetMapping("/list")
    public CommonResult<List<AircraftVo>> list(AircraftBo aircraftBo) {
        List<AircraftVo> list = aircraftService.selectList(aircraftBo);
        return CommonResult.success(list);
    }

    /**
     * 导出机型机号列表
     */
    @SaCheckPermission("als:aircraft:export")
    @Log(title = "机型机号", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public CommonResult<Void> export(HttpServletResponse response, AircraftBo aircraftBo) {
        LoginUser loginUser = LoginHelper.getLoginUser();
        List<AircraftVo> list = aircraftService.selectList(aircraftBo);
        aircraftService.asyncExport(list, "机型机号", loginUser);
        return CommonResult.success();
    }

    /**
     * 导入机型机号列表
     */
    @Log(title = "机型机号", businessType = BusinessType.IMPORT)
    @SaCheckPermission("als:aircraft:import")
    @PostMapping("/importData")
    public CommonResult<Void> importData(MultipartFile file, boolean updateSupport) {
        LoginUser loginUser = LoginHelper.getLoginUser();
        aircraftService.asyncImportData(file, updateSupport, loginUser);
        return CommonResult.success();
    }

    /**
     * 获取机型机号详细信息
     */
    @SaCheckPermission("als:aircraft:query")
    @GetMapping(value = "/{id}")
    public CommonResult<AircraftVo> getInfo(@PathVariable Long id) {
        return CommonResult.success(aircraftService.selectById(id));
    }

    /**
     * 新增机型机号
     */
    @SaCheckPermission("als:aircraft:add")
    @Log(title = "机型机号", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping
    public CommonResult<Void> add(@Validated @RequestBody AircraftBo aircraftBo) {
        boolean inserted = aircraftService.insert(aircraftBo);
        if (!inserted) {
            return CommonResult.fail("新增机型机号记录失败！");
        }
        return CommonResult.success();
    }

    /**
     * 修改机型机号
     */
    @SaCheckPermission("als:aircraft:edit")
    @Log(title = "机型机号", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping
    public CommonResult<Void> edit(@Validated @RequestBody AircraftBo aircraftBo) {
        boolean updated = aircraftService.update(aircraftBo);
        if (!updated) {
            return CommonResult.fail("修改机型机号记录失败!");
        }
        return CommonResult.success();
    }

    /**
     * 删除机型机号
     */
    @SaCheckPermission("als:aircraft:remove")
    @Log(title = "机型机号", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public CommonResult<Void> remove(@PathVariable Long[] ids) {
        boolean deleted = aircraftService.deleteByIds(ids);
        if (!deleted) {
            return CommonResult.fail("删除机型机号记录失败!");
        }
        return CommonResult.success();
    }
}
