package org.eco.als.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import org.eco.als.domain.bo.AlsFaultCaseBo;
import org.eco.als.domain.vo.AlsFaultCaseVo;
import org.eco.als.service.IAlsFaultCaseService;
import org.eco.common.core.core.domain.CommonResult;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.core.core.page.PageResult;
import org.eco.common.log.annotation.Log;
import org.eco.common.log.enums.BusinessType;
import org.eco.common.security.utils.LoginHelper;
import org.eco.common.web.annotation.RepeatSubmit;
import org.eco.common.web.core.BaseController;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

/**
 * 故障案例Controller
 *
 * @author wgk
 * @date 2024-07-18
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/als/faultCase")
public class AlsFaultCaseController extends BaseController {
    @Resource
    private IAlsFaultCaseService alsFaultCaseService;

    /**
     * 查询故障案例列表
     */
    @SaCheckPermission("als:faultCase:list")
    @GetMapping("/list")
    public CommonResult<PageResult<AlsFaultCaseVo>> list(AlsFaultCaseBo alsFaultCaseBo) {
        return CommonResult.success(alsFaultCaseService.selectPage(alsFaultCaseBo));
    }

    /**
     * 导出故障案例列表
     */
    @SaCheckPermission("als:faultCase:export")
    @Log(title = "故障案例", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public CommonResult<Void> export(HttpServletResponse response, AlsFaultCaseBo alsFaultCaseBo) {
        LoginUser loginUser = LoginHelper.getLoginUser();
        List<AlsFaultCaseVo> list = alsFaultCaseService.selectList(alsFaultCaseBo);
        alsFaultCaseService.asyncExport(list, "故障案例", loginUser);
        return CommonResult.success();
    }

    /**
     * 导入故障案例列表
     */
    @Log(title = "故障案例", businessType = BusinessType.IMPORT)
    @SaCheckPermission("als:faultCase:import")
    @PostMapping("/importData")
    public CommonResult<Void> importData(MultipartFile file, boolean updateSupport) {
        LoginUser loginUser = LoginHelper.getLoginUser();
        alsFaultCaseService.asyncImportData(file, updateSupport, loginUser);
        return CommonResult.success();
    }

    /**
     * 获取故障案例详细信息
     */
    @SaCheckPermission("als:faultCase:query")
    @GetMapping(value = "/{id}")
    public CommonResult<AlsFaultCaseVo> getInfo(@PathVariable Long id) {
        return CommonResult.success(alsFaultCaseService.selectById(id));
    }

    /**
     * 新增故障案例
     */
    @SaCheckPermission("als:faultCase:add")
    @Log(title = "故障案例", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping
    public CommonResult<Void> add(@Validated @RequestBody AlsFaultCaseBo alsFaultCaseBo) {
        boolean inserted = alsFaultCaseService.insert(alsFaultCaseBo);
        if (!inserted) {
            return CommonResult.fail("新增故障案例记录失败！");
        }
        return CommonResult.success();
    }

    /**
     * 修改故障案例
     */
    @SaCheckPermission("als:faultCase:edit")
    @Log(title = "故障案例", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping
    public CommonResult<Void> edit(@Validated @RequestBody AlsFaultCaseBo alsFaultCaseBo) {
        boolean updated = alsFaultCaseService.update(alsFaultCaseBo);
        if (!updated) {
            return CommonResult.fail("修改故障案例记录失败!");
        }
        return CommonResult.success();
    }

    /**
     * 删除故障案例
     */
    @SaCheckPermission("als:faultCase:remove")
    @Log(title = "故障案例", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public CommonResult<Void> remove(@PathVariable Long[] ids) {
        boolean deleted = alsFaultCaseService.deleteByIds(ids);
        if (!deleted) {
            return CommonResult.fail("删除故障案例记录失败!");
        }
        return CommonResult.success();
    }
}
