package org.eco.als.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.mybatisflex.core.paginate.Page;
import com.mybatisflex.core.query.QueryWrapper;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.eco.als.domain.AlsFaultCase;
import org.eco.als.domain.bo.AlsFaultCaseBo;
import org.eco.als.domain.vo.AlsFaultCaseImportVo;
import org.eco.als.domain.vo.AlsFaultCaseVo;
import org.eco.als.listener.AlsFaultCaseImportListener;
import org.eco.als.mapper.AlsFaultCaseMapper;
import org.eco.als.service.IAlsFaultCaseService;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.core.core.page.PageResult;
import org.eco.common.core.utils.MapstructUtils;
import org.eco.common.core.utils.bean.BeanUtils;
import org.eco.common.excel.entity.ExcelResultRes;
import org.eco.common.excel.service.IExcelService;
import org.eco.common.orm.core.page.PageQuery;
import org.eco.common.orm.core.service.impl.BaseServiceImpl;
import org.eco.system.domain.bo.ImportExportBo;
import org.eco.system.service.IImportExportService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.Arrays;
import java.util.List;

import static org.eco.als.domain.table.AlsFaultCaseTableDef.ALS_FAULT_CASE;

/**
 * 故障案例Service业务层处理
 *
 * @author wgk
 * @date 2024-07-18
 */
@Service
@Slf4j
public class AlsFaultCaseServiceImpl extends BaseServiceImpl<AlsFaultCaseMapper, AlsFaultCase> implements IAlsFaultCaseService {
    @Resource
    private AlsFaultCaseMapper alsFaultCaseMapper;

    @Resource
    private IExcelService excelService;

    @Resource
    private IImportExportService importExportService;

    @Override
    public QueryWrapper query() {
        return super.query().from(ALS_FAULT_CASE);
    }

    private QueryWrapper buildQueryWrapper(AlsFaultCaseBo alsFaultCaseBo) {
        QueryWrapper queryWrapper = super.buildBaseQueryWrapper();
        queryWrapper.and(ALS_FAULT_CASE.AIRCRAFT_TYPE.eq
            (alsFaultCaseBo.getAircraftType()));
        queryWrapper.and(ALS_FAULT_CASE.UNIT_NAME.like
            (alsFaultCaseBo.getUnitName()));
        queryWrapper.and(ALS_FAULT_CASE.ITEM_NAME.like
            (alsFaultCaseBo.getItemName()));
        queryWrapper.and(ALS_FAULT_CASE.OCCUR_DATE.eq
            (alsFaultCaseBo.getOccurDate()));
        queryWrapper.and(ALS_FAULT_CASE.SYSTEM_NAME.like
            (alsFaultCaseBo.getSystemName()));

        return queryWrapper;
    }

    /**
     * 查询故障案例
     *
     * @param id 故障案例主键
     * @return 故障案例
     */
    @Override
    public AlsFaultCaseVo selectById(Long id) {
        return this.getOneAs(query().where(ALS_FAULT_CASE.ID.eq(id)), AlsFaultCaseVo.class);

    }

    /**
     * 查询故障案例列表
     *
     * @param alsFaultCaseBo 故障案例Bo
     * @return 故障案例集合
     */
    @Override
    public List<AlsFaultCaseVo> selectList(AlsFaultCaseBo alsFaultCaseBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(alsFaultCaseBo);
        return this.listAs(queryWrapper, AlsFaultCaseVo.class);
    }

    /**
     * 分页查询故障案例列表
     *
     * @param alsFaultCaseBo 故障案例Bo
     * @return 分页故障案例集合
     */
    @Override
    public PageResult<AlsFaultCaseVo> selectPage(AlsFaultCaseBo alsFaultCaseBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(alsFaultCaseBo);
        Page<AlsFaultCaseVo> page = this.pageAs(PageQuery.build(), queryWrapper, AlsFaultCaseVo.class);
        return PageResult.build(page);
    }

    /**
     * 新增故障案例
     *
     * @param alsFaultCaseBo 故障案例Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insert(AlsFaultCaseBo alsFaultCaseBo) {
        AlsFaultCase alsFaultCase = MapstructUtils.convert(alsFaultCaseBo, AlsFaultCase.class);

        return this.save(alsFaultCase);//使用全局配置的雪花算法主键生成器生成ID值
    }

    /**
     * 新增故障案例，前台提供主键值，一般用于导入的场合
     *
     * @param alsFaultCaseBo 故障案例Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insertWithPk(AlsFaultCaseBo alsFaultCaseBo) {
        AlsFaultCase alsFaultCase = MapstructUtils.convert(alsFaultCaseBo, AlsFaultCase.class);


        return alsFaultCaseMapper.insertWithPk(alsFaultCase) > 0;//前台传来主键值
    }

    /**
     * 修改故障案例
     *
     * @param alsFaultCaseBo 故障案例Bo
     * @return 结果:true 更新成功，false 更新失败
     */
    @Override
    public boolean update(AlsFaultCaseBo alsFaultCaseBo) {
        AlsFaultCase alsFaultCase = MapstructUtils.convert(alsFaultCaseBo, AlsFaultCase.class);
        if (ObjectUtil.isNotNull(alsFaultCase) && ObjectUtil.isNotNull(alsFaultCase.getId())) {
            boolean updated = this.updateById(alsFaultCase);
            return updated;
        }
        return false;
    }

    @Override
    public void asyncImportData(MultipartFile file, boolean updateSupport, LoginUser loginUser) {
        ExcelResultRes result;
        try {
            String name = file.getOriginalFilename();
            result = excelService.importExcel(file.getInputStream(), name, AlsFaultCaseImportVo.class, new AlsFaultCaseImportListener(updateSupport, loginUser));
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        ImportExportBo bo = new ImportExportBo();
        BeanUtils.copyProperties(result, bo);
        bo.setUpdateBy(loginUser.getUserId());
        bo.setCreateBy(loginUser.getUserId());
        bo.setType("0");
        boolean flag = importExportService.insert(bo);
        if (flag) {
            log.info("异步导入日志写入成功");
        }
    }

    @Override
    public void asyncExport(List<AlsFaultCaseVo> listVo, String sheetName, LoginUser loginUser) {
        ExcelResultRes result = excelService.exportExcel(listVo, sheetName, AlsFaultCaseVo.class);
        ImportExportBo bo = new ImportExportBo();
        BeanUtils.copyProperties(result, bo);
        bo.setUpdateBy(loginUser.getUserId());
        bo.setCreateBy(loginUser.getUserId());
        bo.setType("1");
        boolean flag = importExportService.insert(bo);
        if (flag) {
            log.info("异步导出日志写入成功");
        }
    }

    /**
     * 批量删除故障案例
     *
     * @param ids 需要删除的故障案例主键集合
     * @return 结果:true 删除成功，false 删除失败
     */
    @Transactional
    @Override
    public boolean deleteByIds(Long[] ids) {
        return this.removeByIds(Arrays.asList(ids));
    }

}
